from __future__ import unicode_literals

from itertools import count

from django.db import migrations


def renumber_slots(apps, schema_editor):
    Queue = apps.get_model('register', 'Queue')
    for queue in Queue.objects.all():
        counter = count()
        for slot in queue.slots.order_by('timestamp'):
            slot.monotonic_position = next(counter)
            slot.save()


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0004_add_queue_monotonic_position'),
    ]

    operations = [
        migrations.RunPython(renumber_slots, noop),
    ]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from django.template import Context, Template

from django.contrib.sites.models import Site

from bursary.models import Bursary

SUBJECT_TEMPLATE = Template(
    '[Action needed] Reimbursement process for {{ WAFER_CONFERENCE_NAME }} '
    'travel bursary recipients')

BODY_TEMPLATE = Template("""\
{% autoescape off %}
Dear {{ object.user.get_full_name }},

You have been granted a travel bursary for DebConf, and now is the time to
submit documentation for reimbursement. TL;DR: yes, this email is very long,
but it is all important, you need to read it in full to get travel money back!

Reimbursement for DebConf-related expenses will be processed by Software in the
Public Interest, for all Requesters including Europeans.

The amount you requested is {{ object.travel_bursary }} {{ DEBCONF_BURSARY_CURRENCY }}, and this is therefore the
maximum you'll be able to request for reimbursement.
This does not have to be requested *in* {{ DEBCONF_BURSARY_CURRENCY }}, you can request an equivalent
amount in your home currency.

To process the hundreds of reimbursement requests efficiently and
expeditiously, SPI requires consistency. All requests must include:

 1. an SPI Reimbursement Request Form [1] with accurate banking information as
    SPI uses TransferWise [2]
 2. an Expense Report generated using XE's Travel Expenses Calculator [3]
 3. sufficient documentation substantiating the expense report
 4. a declaration of compliance

And must be submitted to the debconf-reimbursements RT queue (instead of the
normal SPI reimbursement RT queue, listed on [1]).

[1]: https://spi-inc.org/treasurer/reimbursement-form/
[2]: https://transferwise.com
[3]: http://www.xe.com/travel-expenses-calculator/

Requesters must follow these steps:

Step 1: Prepare an Expense Report.
 • use http://www.xe.com/travel-expenses-calculator/ to prepare an expense report
 • enter Your Name
 • set Your Home Currency to the currency of your bank account.
 • leave/set Credit Card @ 3%, Debit Card @ 5%, Foreign Cash @ 5%, Traveller's Cheque @ 2%
 • enter Receipt Details, one row per receipt
   • specify the correct date of the transactions
   • set the currency per transaction to the currency you paid in
 • save as PDF

Step 2: Prepare an SPI Reimbursement Request Form
 • visit https://spi-inc.org/treasurer/reimbursement-form/
 • fill out the form and download the completed PDF
 • ensure your name exactly matches the Expense Report from Step 1
 • e̲n̲s̲u̲r̲e̲ ̲t̲h̲a̲t̲ ̲t̲h̲e̲ ̲b̲a̲n̲k̲i̲n̲g̲ ̲i̲n̲f̲o̲r̲m̲a̲t̲i̲o̲n̲ ̲i̲s̲ ̲a̲c̲c̲u̲r̲a̲t̲e̲ ̲a̲s̲ ̲i̲n̲c̲o̲r̲r̲e̲c̲t̲ ̲d̲e̲t̲a̲i̲l̲s̲ ̲a̲r̲e̲ ̲a̲
   m̲a̲j̲o̲r̲ ̲s̲o̲u̲r̲c̲e̲ ̲o̲f̲ ̲d̲e̲l̲a̲y̲s̲
 • the currency field is the currency of your bank account. Only say {{ DEBCONF_BURSARY_CURRENCY }}
   here if you have a {{ DEBCONF_BURSARY_CURRENCY }} bank account (e.g. because you live there)
 • save as PDF

Step 3: Collect and order your Receipts.
 • collect your receipts in the SAME ORDER as the rows in the Expense Report
 • if paper receipts, scan them with a multi-function device, converting to PDF
 • save as PDF

Step 4: Prepare the Submission Package.
 • collect into a single PDF, the following:
   • from step 2, the SPI Reimbursement Request From
   • from step 1, the Expense Report
   • from step 3, the Ordered Receipts
 • the poppler-utils package includes the pdfunite utility
   • usage: `pdfunite step2.pdf step1.pdf step3a.pdf step3b.pdf ... step3n.pdf {{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf`
   • warning: ensure that you explicitly mention the destination filename
     ({{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf) otherwise
     step3n.pdf is overwritten
   • notice: please name the file exactly as shown, substituting your name as
     entered in the SPI Reimbursement Request Form, for example,
     {{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf
 • e̲n̲s̲u̲r̲e̲ ̲t̲h̲a̲t̲ ̲t̲h̲e̲ ̲e̲n̲t̲i̲r̲e̲ ̲S̲u̲b̲m̲i̲s̲s̲i̲o̲n̲ ̲P̲a̲c̲k̲a̲g̲e̲ ̲c̲a̲n̲ ̲b̲e̲ ̲e̲a̲s̲i̲l̲y̲ ̲u̲n̲d̲e̲r̲s̲t̲o̲o̲d̲ ̲a̲s̲ ̲p̲o̲o̲r̲
   q̲u̲a̲l̲i̲t̲y̲ ̲s̲u̲b̲m̲i̲s̲s̲i̲o̲n̲s̲ ̲a̲r̲e̲ ̲a̲ ̲m̲a̲j̲o̲r̲ ̲s̲o̲u̲r̲c̲e̲ ̲o̲f̲ ̲d̲e̲l̲a̲y̲s̲

Step 5: Email the Submission Package.
 • prepare an email having these attributes
   • from: you
   • To: debconf-reimbursements@rt.spi-inc.org
   • subject: {{ WAFER_CONFERENCE_NAME }} Reimbursement Request for {{ object.user.get_full_name }}
   • attachment: the Submission Package ({{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf) prepared in step 4
   • body: (the text below, please fix the claimed amount in the last line to
            match your actual request, in your local currency)
--- BEGIN TICKET TEXT ---
By submitting this reimbursement request, I declare:
 • that I have accurately prepared an SPI Reimbursement Request Form,
 • that I have prepared an Expense Report using XE's Travel Expense Calculator,
 • that I have attached sufficient documentation substantiating my request,
 • that I seek reimbursement of expenses that are compliant to DebConf policies, and
 • that I have not sought nor will seek reimbursement of these expenses from any other source.

The amount I am claiming for reimbursement is {{ object.travel_bursary }} {{ DEBCONF_BURSARY_CURRENCY }} (out of {{ object.travel_bursary }} {{ DEBCONF_BURSARY_CURRENCY }} allocated).
---- END TICKET TEXT ----

If you have any question, please be sure to contact us at bursaries@debconf.org
and we'll work your issues out.

Thanks for your cooperation!
--\u0020
The DebConf Bursaries team
{% endautoescape %}
""")


class Command(BaseCommand):
    help = 'Send an email to people whose bursary grant expires soon'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually send emails')

    def badger(self, bursary, conference_name, dry_run):
        context = {
            'object': bursary,
            'user': bursary.user.username,
            'to': '%s <%s>' % (bursary.user.get_full_name(),
                               bursary.user.email),
            'short_full_name': bursary.user.get_full_name().replace(' ', '')
                                                           .replace('-', '')
                                                           .replace("'", '')
                                                           .replace('.', ''),
            'DEBCONF_BURSARY_CURRENCY': settings.DEBCONF_BURSARY_CURRENCY,
            'WAFER_CONFERENCE_NAME': conference_name,
        }

        if dry_run:
            print('I would badger {to} (max = {object.travel_bursary})'
                  .format(**context))
            return

        from_email = 'bursaries@debconf.org'
        ctx = Context(context)
        subject = SUBJECT_TEMPLATE.render(ctx)
        body = BODY_TEMPLATE.render(ctx)

        msg = EmailMultiAlternatives(subject, body, to=[context['to']],
                                     from_email=from_email)

        msg.send()

    def handle(self, *args, **options):
        site = Site.objects.get()
        conference_name = site.name.replace(' ', '')
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        to_badger = Bursary.objects.filter(
            request_travel=True,
            travel_status='accepted',
            user__attendee__check_in__pk__isnull=False,
            reimbursed_amount=0,
        ).order_by('user__username')

        for bursary in to_badger:
            self.badger(bursary, conference_name, dry_run)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-04 19:33
from __future__ import unicode_literals

import bursary.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Bursary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('request_food', models.BooleanField()),
                ('request_accommodation', models.BooleanField()),
                ('request_travel', models.BooleanField()),
                ('reason_contribution', models.TextField(blank=True)),
                ('reason_plans', models.TextField(blank=True)),
                ('reason_diversity', models.TextField(blank=True)),
                ('need', models.CharField(blank=True, choices=[('unable', 'Without this funding, I will be absolutely unable to attend'), ('sacrifice', 'Without the requested funding, I will have to make financial sacrifices to attend'), ('inconvenient', 'Without the requested funding, attending will be inconvenient for me'), ('non-financial', 'I am not applying based on financial need')], max_length=16)),
                ('travel_bursary', models.IntegerField(blank=True, null=True)),
                ('travel_from', models.TextField(blank=True)),
                ('food_status', models.CharField(choices=[('submitted', 'The bursary request has been submitted'), ('ranked', 'The bursary request has been ranked'), ('pending', 'The bursary request has been granted and is pending your acceptance'), ('accepted', 'The bursary has been accepted'), ('denied', 'The bursary has been denied'), ('expired', 'The bursary grant has expired'), ('canceled', 'The bursary request has been canceled')], default='submitted', max_length=32)),
                ('food_accept_before', models.DateField(blank=True, null=True)),
                ('accommodation_status', models.CharField(choices=[('submitted', 'The bursary request has been submitted'), ('ranked', 'The bursary request has been ranked'), ('pending', 'The bursary request has been granted and is pending your acceptance'), ('accepted', 'The bursary has been accepted'), ('denied', 'The bursary has been denied'), ('expired', 'The bursary grant has expired'), ('canceled', 'The bursary request has been canceled')], default='submitted', max_length=32)),
                ('accommodation_accept_before', models.DateField(blank=True, null=True)),
                ('travel_status', models.CharField(choices=[('submitted', 'The bursary request has been submitted'), ('ranked', 'The bursary request has been ranked'), ('pending', 'The bursary request has been granted and is pending your acceptance'), ('accepted', 'The bursary has been accepted'), ('denied', 'The bursary has been denied'), ('expired', 'The bursary grant has expired'), ('canceled', 'The bursary request has been canceled')], default='submitted', max_length=32)),
                ('travel_accept_before', models.DateField(blank=True, null=True)),
                ('reimbursed_amount', models.IntegerField(blank=True, default=0)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='bursary', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'bursary request',
            },
        ),
        migrations.CreateModel(
            name='BursaryReferee',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contrib_score', models.DecimalField(blank=True, decimal_places=2, help_text='\n\nDecimal value from 0 to 5; Please try to conform to the following scale for\nyour assessment.\n\n<dl>\n<dt>5 "Must fund"</dt>\n<dd>\nIf this person does not attend DebConf, there will be significant\nnegative impact for DebConf or Debian more generally.\n</dd>\n<dt>4 "Priority funding"</dt>\n<dd>\nThere are clear benefits to Debian or to DebConf of this person attending\ndebconf. This might be an accepted talk that seems particularly important.\n</dd>\n<dt>3 "Good initiative"</dt>\n<dd>\nWe should fund this person because they propose something interesting.\n</dd>\n<dt>2 "Good record"</dt>\n<dd>\nThis person has a record of substantial contribution to Debian.this should\ngenerally be recent contribution, i.e. within the last two years.\n</dd>\n<dt>1 "OK"</dt>\n<dd>\nIf we have budget, I don\'t object to funding this request.\n</dd>\n<dt>0 "Deny"</dt>\n<dd>\nEven if we have budget, I think we should not fund this request.\n</dd>\n</dl>\n', max_digits=3, null=True, validators=[bursary.models.validate_score], verbose_name='Contribution score')),
                ('outreach_score', models.DecimalField(blank=True, decimal_places=2, help_text='\n\nDecimal value from 0 to 5; Please try to assess applicants according to the\nfollowing criteria, and to use the full scale across your (non-null) ratings.\n\n<ul>\n<li>Gender (favoring women (cis, trans, queer), non-binary and genderqueer\nindividuals, as well as trans men)</li>\n<li>Age (favoring individuals over 35 years old)</li>\n<li>Country of origin / ethnic diversity</li>\n<li>Whether the applicant is a newcomer to Debian (yes should rank higher)</li>\n</ul>\n\n', max_digits=3, null=True, validators=[bursary.models.validate_score], verbose_name='Diversity and Inclusion score')),
                ('notes', models.TextField(blank=True, default='', help_text='Let us know how you came to your decision', verbose_name='Notes for evaluation')),
                ('final', models.BooleanField(default=False, verbose_name='Final assessment')),
                ('bursary', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='referees', to='bursary.Bursary')),
                ('referee', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='bursaryreferee',
            unique_together=set([('bursary', 'referee')]),
        ),
    ]



from register.models import Attendee
from wafer.schedule.admin import register_schedule_item_validator
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as N_


def validate_speaker_stay(all_items):
    errors = []
    for item in all_items:
        if not item.talk:
            continue
        for speaker in item.talk.authors.all():
            try:
                attendee = speaker.attendee
            except Attendee.DoesNotExist:
                continue
            for slot in item.slots.all():
                arrival = attendee.arrival
                if arrival and slot.get_start_time() < arrival:
                    errors.append(
                        _('<%(talk)s> scheduled %(scheduled)s; %(speaker)s arrives %(arrival)s') % {
                            'talk': item.talk.title,
                            'speaker': speaker.get_full_name(),
                            'scheduled': slot.get_start_time(),
                            'arrival': arrival,
                        }
                    )
                    break
                departure = attendee.departure
                if departure and slot.end_time > departure:
                    errors.append(
                        _('<%(talk)s> scheduled for %(scheduled)s; %(speaker)s departs %(departure)s') % {
                            'talk': item.talk.title,
                            'speaker': speaker.get_full_name(),
                            'scheduled': slot.end_time,
                            'departure': departure,
                        }
                    )
                    break
    return errors


register_schedule_item_validator(
    validate_speaker_stay,
    'speaker_not_at_conference',
    N_('Talk scheduled when speaker is not at the conference')
)



from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.crypto import get_random_string

from wafer.talks.models import Talk, TalkUrl


def provision_urls_setting():
    """Lookup the DEBCONF_TALK_PROVISION_URLS setting"""
    return getattr(settings, 'DEBCONF_TALK_PROVISION_URLS', {})


def create_online_service_urls(talk, regenerate=False):
    """Generate URLs for any online services associated with the Talk

    e.g. Jitsi Meet rooms or Etherpads
    """
    for key, config in provision_urls_setting().items():
        if talk.urls.filter(description=key).exists() and not regenerate:
            continue
        url = config['pattern'].format(
            id=talk.talk_id,
            slug=talk.slug,
            secret16=get_random_string(length=16))
        TalkUrl.objects.update_or_create(
            talk=talk,
            description=key,
            public=config.get('public', True),
            defaults={'url': url})


@receiver(post_save, sender=Talk)
def create_online_service_urls_reciever(sender, **kwargs):
    talk = kwargs['instance']
    if talk.accepted:
        create_online_service_urls(talk)



from django.db import models

from register.fields import RequiredCountries
from register.models.attendee import Attendee


class Address(models.Model):
    ROLE_CHOICES = (
        ('billing', 'Billing'),
        ('shipping', 'Shipping'),
    )
    attendee = models.ForeignKey(Attendee, related_name='addresses',
        on_delete=models.CASCADE)
    role = models.CharField(max_length=32, choices=ROLE_CHOICES)
    contact = models.CharField(max_length=50)
    company = models.CharField(max_length=50, blank=True)
    line1 = models.CharField(max_length=50)
    line2 = models.CharField(max_length=50, blank=True)
    city = models.CharField(max_length=50)
    province = models.CharField(max_length=50)
    postal_code = models.CharField(max_length=32, blank=True)
    country = models.CharField(max_length=2)
    phone_number = models.CharField(max_length=32, blank=True)

    class Meta:
        unique_together = ('attendee', 'role')

    @property
    def country_name(self):
        return RequiredCountries().name(self.country)

    @property
    def formatted_address(self):
        output = [
            self.contact,
            f'Phone: {self.phone_number}',
            self.company,
            self.line1,
            self.line2,
            self.city,
            self.province,
            self.postal_code,
            self.country_name,
        ]
        return '\n'.join(line for line in output if line)



from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.models import User
from django.http import Http404
from django.views.generic.detail import DetailView

from front_desk.views import CheckInPermissionMixin
from register.models import Accomm, Attendee, Food, Meal

from register.dates import meals as dc_meals


def attendee_context(context, attendee):
    dates = sorted(set(date for meal, date in dc_meals(orga=True)))
    meal_labels = ['Breakfast', 'Lunch', 'Dinner']

    try:
        food = attendee.food
    except Food.DoesNotExist:
        food = None

    meals = []
    for meal, meal_label in Meal.MEALS.items():
        cur_meal = []
        meal_dates = []

        if food != None:
            meal_dates = [entry.date
                          for entry in food.meals.filter(meal=meal)]

        for d in dates:
            cur_meal.append((d, d in meal_dates))
        meals.append((meal_label, cur_meal))

    try:
        accomm = attendee.accomm
    except Accomm.DoesNotExist:
        accomm = None

    nights = []
    night_dates = []
    if accomm != None:
        night_dates = [entry.date
                      for entry in accomm.nights.all()]

    for d in dates:
        nights.append((d, d in night_dates))

    context.update({
        'dates': dates,
        'meal_labels': meal_labels,
        'meals': meals,
        'nights': nights,
    })
    return context


class OwnBadge(LoginRequiredMixin, DetailView):
    template_name = 'badges/single_badge.html'

    def get_object(self):
        return self.request.user

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        try:
            attendee = self.object.attendee
        except Attendee.DoesNotExist:
            messages.warning(self.request, 'Only registered users have badges')
            raise Http404()

        return attendee_context(context, attendee)


class CheckInBadgeView(CheckInPermissionMixin, DetailView):
    template_name = 'badges/single_badge.html'

    model = User
    slug_field = 'username'
    slug_url_kwarg = 'username'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        attendee = self.object.attendee

        return attendee_context(context, attendee)



# -*- coding: utf-8 -*-
import datetime

from django.contrib.sites.models import Site
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.template import Context, Template

from bursary.models import Bursary

SUBJECT_TEMPLATE = Template(
    "[Action needed] Your {{WAFER_CONFERENCE_NAME}} travel bursary grant "
    "expires in {{days}} day{{days|pluralize}}!")

BODY_TEMPLATE = Template("""\
Dear {{object.user.get_full_name}},

Your travel bursary request has been granted by the bursaries team, and you
have not confirmed it yet.

You need to do so before {{ object.travel_accept_before|date }} at 23:59 UTC,
or you will no longer be eligable for this bursary. The budget we have
allocated for you will be redistributed to other applicants.

Please log into the website at https://{{ domain }}/bursary/ to update
your status.

If you're unable to do so before the deadline, let us know by replying to this
message and we'll work something out.

Thanks!
--\u0020
The DebConf Bursaries team
""")


class Command(BaseCommand):
    help = 'Send an email to people whose bursary grant expires soon'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually send emails')
        parser.add_argument('--days-before', '-d', metavar='X,Y',
                            default='6,3,1',
                            help='Send reminders for bursaries expiring X and '
                                 'Y days in the future (comma-separated list, '
                                 'defaults to 6,3,1)')


    def badger(self, bursary, dry_run):
        site = Site.objects.get()
        conference_name = site.name.replace(' ', '')

        context = {
            'WAFER_CONFERENCE_NAME': conference_name,
            'domain': site.domain,
            'object': bursary,
            'user': bursary.user.username,
        }

        delta = bursary.travel_accept_before - datetime.datetime.today().date()
        context['days'] = delta.days

        if dry_run:
            print('I would badger {user} (expiry in {days} days)'
                  .format(**context))
            return

        from_email = 'bursaries@debconf.org'
        to = bursary.user.email
        subject = SUBJECT_TEMPLATE.render(Context(context))
        body = BODY_TEMPLATE.render(Context(context))

        msg = EmailMultiAlternatives(subject, body, to=[to],
                                     from_email=from_email)

        msg.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        days_before = [int(x) for x in options['days_before'].split(',')]

        dates = [
            (datetime.datetime.today().date()
             + datetime.timedelta(days=days))
            for days in days_before
        ]

        if dry_run:
            for date in dates:
                print('Will badger for expiry on %s' % date)

        to_badger = Bursary.objects.filter(
            request_travel=True,
            travel_status='pending',
            travel_accept_before__in=dates,
        )

        for bursary in to_badger:
            self.badger(bursary, dry_run)



from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.contrib.sites.models import Site
from django.template import Context, Template

from bursary.models import Bursary


SUBJECT = '{{ WAFER_CONFERENCE_NAME }}: Registration Automatically Cancelled'
TXT = '''Dear {{ name }},

We're sorry that we were unable to grant you travel assistance for
{{ WAFER_CONFERENCE_NAME }}.

When you applied for your travel bursary, you informed us that, without the
bursary, you'd be unable to attend the conference. As a result, we have
automatically cancelled your registration.

If you still wish to attend {{ WAFER_CONFERENCE_NAME }}, please go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/>
and fill out every page of the form.

Thanks for your interest in attending,
The {{ WAFER_CONFERENCE_NAME }} Registration Team
'''


class Command(BaseCommand):
    help = ("Cancel the registration for attendees without travel bursaries "
            "if they listed their level of need as critical")

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')
        parser.add_argument('--site', type=int, default=1,
                            help='Django site ID, default: 1')

    def badger(self, user, dry_run, site):
        context = Context({
            'name': user.userprofile.display_name(),
            'WAFER_CONFERENCE_DOMAIN': site.domain,
            'WAFER_CONFERENCE_NAME': site.name,
        })

        txt = Template(TXT).render(context)
        subject = Template(SUBJECT).render(context)
        to = user.email
        if dry_run:
            print('I would badger:', to)
            return
        email_message = EmailMultiAlternatives(subject, txt, to=[to])
        email_message.send()

        user.attendee.delete()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        site = Site.objects.get(id=1)
        for bursary in Bursary.objects.filter(
                request_travel=True,
                need='unable',
                travel_status__in=('expired', 'denied', 'canceled')):
            user = bursary.user
            if user.userprofile.is_registered():
                self.badger(user, dry_run, site)



from register.forms.childcare import ChildCareForm
from register.models.childcare import ChildCare
from register.views.core import RegisterStep


class ChildCareView(RegisterStep):
    template_name = 'register/page/childcare.html'
    title = 'Child Care'
    form_class = ChildCareForm

    def get_initial(self):
        user = self.request.user
        initial = {}

        try:
            child_care = user.attendee.child_care
        except ChildCare.DoesNotExist:
            return initial

        initial['child_care'] = True

        for field in child_care._meta.get_fields():
            if field.is_relation:
                continue
            initial[field.name] = getattr(child_care, field.name)

        return initial

    def form_valid(self, form):
        attendee = self.request.user.attendee
        data = form.cleaned_data.copy()

        if not data.pop('child_care'):
            ChildCare.objects.filter(attendee=attendee).delete()
            return super().form_valid(form)

        child_care, created = ChildCare.objects.update_or_create(
            attendee=attendee, defaults=data)
        attendee.child_care = child_care

        return super().form_valid(form)



# Generated by Django 2.2.27 on 2022-04-20 15:27

from django.conf import settings
from django.db import migrations, models


def default_accomm():
    options = list(settings.PRICES['accomm'].items())
    options.sort(key=lambda kv:
            kv[1].get('price', not kv[1].get('bursary', False)))
    if options:
        return options[0][0]
    return ''


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0014_attendee_confirm_covid_tests'),
    ]

    operations = [
        migrations.AddField(
            model_name='accomm',
            name='option',
            field=models.CharField(default=default_accomm(), max_length=32),
        ),
    ]



from django.core.management.base import BaseCommand

from register.dates import nights, meals
from register.models import AccommNight, Meal


class Command(BaseCommand):
    help = 'Create Meal and Night objects in the DB'

    def handle(self, *args, **options):
        for night in nights(orga=True):
            AccommNight.objects.get_or_create(date=night)
        for meal, date in meals(orga=True):
            Meal.objects.get_or_create(meal=meal, date=date)



import os
from wafer.settings import *

INSTALLED_APPS = (
    'django_countries',
    'compressor',
    'debconf.themes.empty',
) + INSTALLED_APPS

try:
    import django_extensions
    INSTALLED_APPS += ('django_extensions',)
except ImportError:
    pass

ALLOWED_HOSTS = ["*"]

TEMPLATES[0]['OPTIONS']['context_processors'] += (
    'debconf.context_processors.expose_settings',
    'debconf.context_processors.is_it_debconf',
)

BAKERY_VIEWS += (
    'debconf.views.RobotsView',
    'debconf.views.IndexView',
    'debconf.views.DebConfScheduleView',
    'debconf.views.StatisticsView',
    'debconf.views.ContentStatisticsView',
    'register.views.statistics.StatisticsView',
    'volunteers.views.VolunteerStatisticsView',
)

SANDBOX = False
DEBCONF_ONLINE = False
DEBCONF_BILLING_CURRENCY = 'USD'
DEBCONF_BILLING_CURRENCY_SYMBOL = '$'
DEBCONF_BURSARY_CURRENCY = 'USD'
DEBCONF_BURSARY_CURRENCY_SYMBOL = '$'
DEBCONF_LOCAL_CURRENCY = 'USD'
DEBCONF_LOCAL_CURRENCY_SYMBOL = '$'

markdown_kwargs = {
    'extensions': [
        'markdown.extensions.smarty',
        'markdown.extensions.tables',
        'markdown.extensions.toc',
        'mdx_linkify.mdx_linkify',
        'mdx_staticfiles',
    ],
    'output_format': 'html5',
}
MARKITUP_FILTER = ('wafer.markdown.bleached_markdown', markdown_kwargs)
WAFER_PAGE_MARKITUP_FILTER = ('markdown.markdown', markdown_kwargs)
MARKITUP_SET = 'markitup/sets/markdown/'
JQUERY_URL = 'vendor/jquery/jquery.js'

COMPRESS_PRECOMPILERS = (
    ('text/x-scss', 'django_libsass.SassCompiler'),
)

STATICFILES_FINDERS = (
    'compressor.finders.CompressorFinder',
) + STATICFILES_FINDERS

SITE_DESCRIPTION = os.getenv("SITE_DESCRIPTION")
SITE_AUTHOR = os.getenv("SITE_AUTHOR")

WAFER_SSO = ('gitlab',)
WAFER_GITLAB_HOSTNAME = 'salsa.debian.org'
WAFER_GITLAB_CLIENT_ID = os.getenv('WAFER_GITLAB_CLIENT_ID')
WAFER_GITLAB_CLIENT_SECRET = os.getenv('WAFER_GITLAB_CLIENT_SECRET')

WAFER_VIDEO_REVIEWER = False

DEBCONF_VENUE_STREAM_HLS_URL = "https://onsite.live.debconf.org/redir/live/main.m3u8"
DEBCONF_VENUE_STREAM_RTMP_URL = "rtmp://onsite.live.debconf.org/redir/front/main_{quality}"
DEBCONF_VENUE_IRC_CHANNELS = ["#debconf"]



import re
from django import template
from django.conf import settings

from debconf.talk_urls import provision_urls_setting

register = template.Library()

debconf = {
    'BILLING_CURRENCY': settings.DEBCONF_BILLING_CURRENCY,
    'BURSARY_CURRENCY': settings.DEBCONF_BURSARY_CURRENCY,
    'LOCAL_CURRENCY': settings.DEBCONF_LOCAL_CURRENCY,
}

@register.simple_tag
def debconf_setting(key):
    return debconf[key]


# These colors were taken by picking the lightest colors from
# https://en.wikipedia.org/wiki/Web_colors#X11_color_names then shuffling the
# list
colors = [
    ('#FFE4E1', 'MistyRose'),
    ('#F0FFF0', 'Honeydew'),
    ('#00FFFF', 'Cyan'),
    ('#E0FFFF', 'LightCyan'),
    ('#00FF00', 'Lime'),
    ('#D2B48C', 'Tan'),
    ('#FFD700', 'Gold'),
    ('#D3D3D3', 'LightGray'),
    ('#B0C4DE', 'LightSteelBlue'),
    ('#E6E6FA', 'Lavender'),
    ('#00BFFF', 'DeepSkyBlue'),
    ('#F5DEB3', 'Wheat'),
    ('#90EE90', 'LightGreen'),
    ('#FFB6C1', 'LightPink'),
    ('#FFA07A', 'LightSalmon'),
    ('#AFEEEE', 'PaleTurquoise'),
    ('#DDA0DD', 'Plum'),
    ('#FFFFE0', 'LightYellow'),
    ('#F0E68C', 'Khaki'),
]


@register.simple_tag
def debconf_track_color(track):
    if track:
        return colors[track.id % len(colors)][0]


@register.simple_tag
def is_among_authors(user, talk):
    return talk._is_among_authors(user)


@register.simple_tag
def talk_has_private_urls(talk):
    return talk.urls.filter(public=False).exists()


def venue_name(venue):
    return re.sub(r"[^a-z0-9-]+", "-", venue.name.lower())

@register.simple_tag
def hls_stream_url(venue):
    url = settings.DEBCONF_VENUE_STREAM_HLS_URL
    return url.format(name=venue_name(venue))

@register.simple_tag
def rtmp_stream_url(venue, quality):
    url = settings.DEBCONF_VENUE_STREAM_RTMP_URL
    return url.format(name=venue_name(venue), quality=quality)


@register.simple_tag
def irc_channels(venue):
    return [
        ch.format(name=venue_name(venue)) for ch in settings.DEBCONF_VENUE_IRC_CHANNELS
    ]



# Generated by Django 2.2.14 on 2020-07-18 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0009_backfill_registration_timestamps'),
    ]

    operations = [
        migrations.AddField(
            model_name='attendee',
            name='shipping_address',
            field=models.TextField(blank=True),
        ),
    ]



import json
import logging

from django.contrib.auth.mixins import LoginRequiredMixin
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.serializers.json import DjangoJSONEncoder
from django.db import transaction
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.functional import cached_property
from django.views.generic import TemplateView
from django.views.generic.edit import FormView

from register.models.attendee import Attendee
from bursary.models import Bursary


class RegisterStep(LoginRequiredMixin, FormView):
    template_name = 'register/form.html'

    def dispatch(self, request, *args, **kwargs):
        if not settings.WAFER_REGISTRATION_OPEN:
            return redirect('register-closed')
        return super().dispatch(request, *args, **kwargs)

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        return super().post(request, *args, **kwargs)

    @cached_property
    def register_step(self):
        from register.views import STEPS
        return STEPS.index(self.__class__)

    @cached_property
    def completed_register_steps(self):
        try:
            return self.request.user.attendee.completed_register_steps
        except ObjectDoesNotExist:
            return 0

    def get_context_data(self, **kwargs):
        from register.views import STEPS

        count = len(STEPS)
        context = super().get_context_data(**kwargs)
        steps = {
            'count': count,
            'percentage': 100 * (self.register_step + 1) / count,
            'step0': self.register_step,
            'step1': self.register_step + 1,
            'links': [],
        }
        for i, step in enumerate(STEPS):
            steps['links'].append({
                'accessible': self.completed_register_steps >= i,
                'current': i == self.register_step,
                'index': i,
                'title': step.title,
                'url': reverse('register-step-{}'.format(i)),
            })

        if self.register_step > 0:
            steps['prev'] = self.register_step - 1

        context['register'] = {
            'steps': steps,
            'title': self.title,
        }
        return context

    def get_success_url(self):
        if self.completed_register_steps < self.register_step > 0:
            attendee = self.request.user.attendee
            attendee.completed_register_steps = self.register_step
            attendee.save()

        next_step = self.register_step + 1

        wizard_goto_step = self.request.POST.get('wizard_goto_step')
        if wizard_goto_step:
            next_step = int(wizard_goto_step)

        return reverse('register-step-{}'.format(next_step))

    def form_valid(self, form):
        log = logging.getLogger('register.step')
        log.debug('Form received: user=%s step=%s data=%s',
                  self.request.user.username, self.register_step,
                  json.dumps(form.cleaned_data, cls=DjangoJSONEncoder))
        return super().form_valid(form)


class ClosedView(TemplateView):
    template_name = 'register/closed.html'


class UnRegisterView(LoginRequiredMixin, TemplateView):
    template_name = 'register/unregister.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        user = self.request.user
        context.update({
            'registered': Attendee.objects.filter(user=user).exists(),
            'bursary': Bursary.objects.filter(user=user).exists(),
        })
        return context

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        user = self.request.user
        user.attendee.delete()

        Bursary.objects.filter(user=user).update(
            request_accommodation=False,
            request_food=False,
            request_travel=False)

        return self.get(request, *args, **kwargs)



from collections import defaultdict
from datetime import timedelta

from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.http import JsonResponse
from django.utils import timezone

from bakery.views import BuildableTemplateView
from wafer.talks.models import Talk, TalkType, Track
from wafer.schedule.views import ScheduleView
from wafer.schedule.models import Venue, ScheduleItem, Slot
from wafer.pages.models import Page


class RobotsView(BuildableTemplateView):
    build_path = 'robots.txt'
    template_name = 'debconf/robots.txt'
    content_type = 'text/plain; charset=UTF-8'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['SANDBOX'] = settings.SANDBOX
        return context


class DebConfScheduleView(ScheduleView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tracks'] = Track.objects.all()
        return context


def get_current_slot():
    now = timezone.now()

    for slot in Slot.objects.all():
        start = slot.start_time
        end = slot.end_time
        if start <= now and now < end:
            return slot


class IndexView(BuildableTemplateView):
    build_path = 'index.html'
    template_name = 'wafer/index.html'

    def get_context_data(self, *args, **kwargs):
        context_data = super().get_context_data(*args, **kwargs)

        venues = Venue.objects.filter(video=True)

        venue_blocks = [{'venue': venue} for venue in venues]

        current_slot = get_current_slot()
        if current_slot:
            events = ScheduleItem.objects.filter(slots=current_slot)
            for event in events:
                for blk in venue_blocks:
                    if event.venue == blk['venue']:
                        slots = list(event.slots.all())
                        blk['event'] = event
                        blk['start_time'] = slots[0].get_formatted_start_time()
                        blk['end_time'] = slots[-1].get_formatted_end_time()

        context_data['venue_blocks'] = venue_blocks

        try:
            context_data['page'] = Page.objects.get(slug="index", parent=None)
        except Page.DoesNotExist:
            pass

        return context_data


class StatisticsView(BuildableTemplateView):
    build_path = 'statistics/index.html'
    template_name = 'debconf/statistics.html'


class ContentStatisticsView(BuildableTemplateView):
    build_path = 'talks/statistics/index.html'
    template_name = 'debconf/content_statistics.html'
    cache_key = 'debconf:content_statistics'
    cache_timeout = 30*60 if not settings.DEBUG else 10

    def get_context_data(self, **kwargs):
        retval = cache.get(self.cache_key)
        if retval:
            return retval

        talks_submitted = Talk.objects.count()
        talks_reviewed = Talk.objects.filter(
            reviews__isnull=False).distinct().count()
        talks_scheduled = Talk.objects.filter(
            scheduleitem__isnull=False).distinct().count()

        minutes_of_content = 0
        for si in ScheduleItem.objects.filter(talk__isnull=False):
            duration = si.get_duration()
            minutes_of_content += duration['minutes'] + duration['hours'] * 60
        hours_of_content = minutes_of_content / 60

        concurrency_by_hour = defaultdict(int)
        for slot in Slot.objects.all():
            hour = slot.start_time.replace(
                minute=0, second=0, microsecond=0)
            concurrency_by_hour[hour] = max(concurrency_by_hour[hour],
                                            slot.scheduleitem_set.count())

        hours_of_concurrency = []
        if concurrency_by_hour:
            hours_of_concurrency = [
                (concurrency, sum(
                    1 for hour, hour_concurrency in concurrency_by_hour.items()
                    if hour_concurrency == concurrency))
                for concurrency in range(max(concurrency_by_hour.values()) + 1)]

        talks_by_track = {}
        for track in Track.objects.all():
            talks_by_track[track.name] = {
                'submitted': track.talk_set.count(),
                'scheduled': track.talk_set.filter(
                    scheduleitem__isnull=False).count(),
            }

        talks_by_type = {}
        for type_ in TalkType.objects.all():
            talks_by_type[type_.name] = {
                'submitted': type_.talk_set.count(),
                'scheduled': type_.talk_set.filter(
                    scheduleitem__isnull=False).count(),
            }

        countries = {}
        for talk in Talk.objects.filter(status__in=('A', 'P')).prefetch_related('authors'):
            for author in talk.authors.all():
                try:
                    if (settings.WAFER_USER_IS_REGISTERED
                            == 'register.models.user_is_registered'):
                        country = author.attendee.country_name
                    elif (settings.WAFER_USER_IS_REGISTERED
                            == 'minidebconf.models.is_registered'):
                        country = author.registration.country.name
                    else:
                        country = 'Unknown'
                except ObjectDoesNotExist:
                    country = 'Not registered yet'
                countries.setdefault(country, 0)
                countries[country] += 1
        speakers_by_country = sorted(countries.items(), key=lambda i: -i[1])


        retval = {
            'talks_submitted': talks_submitted,
            'talks_reviewed': talks_reviewed,
            'talks_scheduled': talks_scheduled,
            'hours_of_content': hours_of_content,
            'hours_of_concurrency': hours_of_concurrency,
            'talks_by_track': talks_by_track,
            'talks_by_type': talks_by_type,
            'speakers_by_country': speakers_by_country,
        }

        cache.set(self.cache_key, retval, self.cache_timeout)
        return retval


def now_or_next(request, venue_id):
    talk_now = False
    talk_next = False
    reload_seconds = 60
    talk = None

    now = timezone.now()
    # now
    try:
        item = ScheduleItem.objects.filter(
            venue=venue_id,
            talk__isnull=False,
            slots__start_time__lte=now,
            slots__end_time__gte=now,
        ).order_by("slots__start_time")[0]
        duration = item.get_duration_minutes()
        end = item.get_start_datetime() + timedelta(minutes=duration)
        reload_seconds = (end - timezone.now()).total_seconds()

        talk_now = True
        talk = item.talk_id
    except IndexError:
        pass

    if not talk:
        # next
        try:
            item = ScheduleItem.objects.filter(
                venue=venue_id,
                talk__isnull=False,
                slots__start_time__gte=now
            ).order_by("slots__start_time")[0]
            reload_seconds = (item.get_start_datetime() - timezone.now()).total_seconds()
            talk_next = True
            talk = item.talk_id
        except IndexError:
            pass

    return JsonResponse({
        "now": talk_now,
        "next": talk_next,
        "reload_seconds": reload_seconds,
        "talk": talk,
    })



import logging

from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings

import requests

from register.forms.attendee import ContactInformationForm
from register.models.attendee import Attendee
from register.views.core import RegisterStep

log = logging.getLogger(__name__)


class ContactInformationView(RegisterStep):
    title = 'Contact Information'
    form_class = ContactInformationForm

    def get_initial(self):
        user = self.request.user
        initial = {
            'name': user.get_full_name(),
            'nametag_3': user.username,
            'email': user.email,
            'phone': user.userprofile.contact_number,
        }

        try:
            attendee = user.attendee
            for field in attendee._meta.get_fields():
                if field.is_relation:
                    continue
                initial[field.name] = getattr(attendee, field.name)
        except ObjectDoesNotExist:
            pass

        return initial

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data.copy()

        name = data.pop('name')
        if user.get_full_name() != name:
            names = name.split(None, 1)
            if len(names) == 1:
                names.append('')
            user.first_name, user.last_name = names

        email_changed = user.email != data['email']
        user.email = data.pop('email')

        user.save()

        user.userprofile.contact_number = data.pop('phone')
        user.userprofile.save()

        if not settings.SANDBOX:
            self.subscribe_to_lists(data, email_changed)

        if settings.DEBCONF_ONLINE:
            data.setdefault('announce_me', False)

        user.attendee, created = Attendee.objects.update_or_create(
            user=user, defaults=data)

        return super().form_valid(form)

    def subscribe_to_lists(self, data, email_changed):
        user = self.request.user
        try:
            attendee = user.attendee
        except Attendee.DoesNotExist:
            attendee = Attendee()

        subscribe = []
        for list_ in ('announce', 'discuss'):
            key = 'register_{}'.format(list_)
            request_subscription = data[key]
            already_registered = getattr(attendee, key)
            if request_subscription and (
                    email_changed or not already_registered):
                subscribe.append('debconf-{}'.format(list_))
        if subscribe:
            requests.post(
                'https://lists.debian.org/cgi-bin/subscribe.pl',
                data={
                    'user_email': user.email,
                    'subscribe': subscribe,
                })
            log.info('Subscribed %s to %r', user.email, subscribe)



import yaml
from datetime import datetime, timedelta
from django.utils import timezone
from django.core.management.base import BaseCommand


from volunteers.models import TaskCategory, TaskTemplate, Task
from wafer.schedule.models import Venue


class Command(BaseCommand):
    help = 'Creates volunteer tasks from a YAML file'

    def add_arguments(self, parser):
        parser.add_argument('FILE', type=open)

    def handle(self, *args, **options):
        data = yaml.safe_load(options['FILE'])

        created_items = {
            'categories': 0,
            'tasks': 0,
            'templates': 0,
            'venues': 0,
        }
        for item in data['tasks']:
            category, created = TaskCategory.objects.get_or_create(
                name=item['category'])
            if created:
                print('Created category:', category)
                created_items['categories'] += 1

            template, created = TaskTemplate.objects.update_or_create(
                name=item['name'],
                defaults={
                    'nbr_volunteers_min': item.get('nbr_volunteers_min', None),
                    'nbr_volunteers_max': item.get('nbr_volunteers_max', None),
                    'description': item.get('description', None),
                    'video_task': item.get('video_task', False),
                    'required_permission': item.get('required_permission', None),
                })
            if created:
                print('Created template:', template)
                created_items['templates'] += 1

            if 'venues' in item:
                venues = []
                for v in item['venues']:
                    venue, created = Venue.objects.get_or_create(name=v)
                    if created:
                        print('Created venue:', venue)
                        created_items['venues'] += 1
                    venues.append(venue)
            else:
                venues = [None]

            if item.get('video_task', False):
                if 'specs' in item or 'days' in item:
                    raise Exception("Tasks can't have video_task and specs/days")
                continue

            task_specs = []
            if 'days' in item:
                task_specs.append(item)
            if 'specs' in item:
                task_specs.extend(item['specs'])

            for spec in task_specs:
                for day in spec['days']:
                    for hour in spec['hours']:
                        startstr = hour['start']
                        start = timezone.make_aware(
                            datetime.strptime('%s %s' % (day, startstr), '%Y-%m-%d %H:%M')
                        )
                        if 'end' in hour:
                            endstr = hour['end']
                            end = timezone.make_aware(
                                datetime.strptime('%s %s' % (day, endstr), '%Y-%m-%d %H:%M')
                            )
                        else:
                            duration = hour['duration']
                            end = start + timedelta(minutes=duration)

                        for venue in venues:
                            task, created = Task.objects.get_or_create(
                                template=template,
                                start=start,
                                end=end,
                                venue=venue,
                            )
                            if created:
                                print('Created task:', task)
                                created_items['tasks'] += 1
        print('Created: {categories} categories, {templates} templates, '
              '{venues} venues, and {tasks} tasks.'.format(**created_items))



import logging

from django.conf import settings

import stripe
from stripe import PaymentIntent

from invoices.models import Invoice

stripe.api_key = settings.STRIPE_SECRET_KEY
log = logging.getLogger(__name__)


def build_metadata(invoice):
    """Generate metadata that we can use in our books"""
    metadata = {
        'reference_number': invoice.reference_number,
    }
    for category, value in invoice.income_breakdown().items():
        if value:  # Follow-up invoices are likely to contain 0 values, strip
            metadata[f'breakdown_{category}'] = float(value)
    return metadata


def payment_intent(invoice):
    ref = invoice.reference_number
    total_cents = int(invoice.total * 100)
    if invoice.transaction_id:
        intent = PaymentIntent.retrieve(invoice.transaction_id)
    else:
        log.info('Creating Stripe PaymentIntent for %s', ref)
        intent = PaymentIntent.create(
            amount=total_cents,
            currency=settings.DEBCONF_BILLING_CURRENCY,
            description='DebConf Invoice {}'.format(ref),
            metadata=build_metadata(invoice),
        )
        invoice.transaction_id = intent.id
        invoice.save()
    if intent.status == 'succeeded':
        log.warn('Stripe PaymentIntent unexpectedly succeeded for %s', ref)
        payment_received(invoice, intent)
        return intent
    if intent.amount != total_cents:
        log.info('Updating PaymentIntent amount for %s', ref)
        intent = PaymentIntent.modify(intent.id, amount=total_cents)
    return intent


def payment_intent_succeeded(intent, ignore_unrelated=False):
    """A payment_intent has succeeded"""
    reference_number = intent.metadata.get('reference_number')
    try:
        invoice = Invoice.objects.get(reference_number=reference_number,
                                      transaction_id=intent.id)
    except Invoice.DoesNotExist:
        if not ignore_unrelated:
            raise
        log.info('Ignoring payment_intent.created for unrelated intent %s',
                 intent.id)
        return
    payment_received(invoice, intent)


def payment_received(invoice, intent):
    total_cents = int(invoice.total * 100)
    if intent.amount != total_cents:
        log.error('Received amount $%s (%s) does not match invoice $%s (%s)',
                  intent.amount / 100.0, intent.id, invoice.total, invoice.id)
        return

    if invoice.status == 'paid':
        # We expect to race between re-rendering the invoice and receiving a
        # webhook.
        return
    if invoice.status not in ('new', 'pending'):
        log.error('Refusing to mark invoice %s as paid (%s) when status is %s',
                  invoice.id, intent.id, invoice.status)
        return

    invoice.status = 'paid'
    invoice.transaction_id = intent.id
    invoice.save()

    if invoice.compound:
        for line in invoice.lines.all():
            reference = line.reference.split('#', 1)[1]
            child_invoice = Invoice.objects.get(reference_number=reference)
            child_invoice.status = 'paid'
            child_invoice.transaction_id = intent.id
            child_invoice.save()


def charge_refunded(charge, ignore_unrelated=False):
    """A payment has been refunded"""
    if not charge.payment_intent:
        log.error('charge.refunded (%s) for charge without intent', charge.id)
        return
    intent = PaymentIntent.retrieve(charge.payment_intent)
    reference_number = intent.metadata.get('reference_number')
    try:
        invoice = Invoice.objects.get(reference_number=reference_number,
                                      transaction_id=intent.id,
                                      status__in=('paid', 'pending'))
    except Invoice.DoesNotExist:
        if not ignore_unrelated:
            raise
        log.info('Ignoring charge.refunded for unrelated intent %s',
                 intent.id)
        return
    invoice.status = 'refunded'
    invoice.save()


def dispute_filed(dispute, ignore_unrelated=False):
    """A charge has been disputed"""
    if not dispute.payment_intent:
        log.error('charge.dispute.created (%s) for charge without intent',
                  dispute.id)
        return
    intent = PaymentIntent.retrieve(dispute.payment_intent)
    reference_number = intent.metadata.get('reference_number')
    try:
        invoice = Invoice.objects.get(reference_number=reference_number,
                                      transaction_id=intent.id,
                                      status__in=('paid', 'pending'))
    except Invoice.DoesNotExist:
        if not ignore_unrelated:
            raise
        log.info('Ignoring charge.dispute.created for unrelated intent %s',
                 intent.id)
        return
    invoice.status = 'disputed'
    invoice.save()
